function _GCC_LibraryExists(exe, lib)
    local cmd = exe .. " .temp.c -o .temp -l" .. lib

    cmd = cmd .. StrJoin(GetLibraryDirs(), " \"-L", "\"")

    file = io.open(".temp.c", "w")
    file:write("int main(int argc, char *argv[]) { return 0; }")
    file:close()

    if Execute(cmd, false) == 0 then
        os.remove(".temp.c")
        os.remove(".temp")

        return true
    end

    os.remove(".temp.c")
    os.remove(".temp")

    return false
end

function _GCC_Deps(exe, file)
    local cmd = exe .. " -M -MM -MG " .. file

    cmd = cmd .. StrJoin(GetIncludeDirs(), " \"-I", "\"")
    cmd = cmd .. StrJoin(GetDefinitions(), " \"-D", "\"")

    local ret, str = Execute(cmd, false)
    local files = {}

    local lines = StrSplit("\n", str)
    i = 1
    line = lines[i]
    while line ~= nil do
        local sections = StrSplit(" ", line)
        j = 1
        if i == 1 then
            j = 3
        end

        section = sections[j]
        while section ~= nil do
            if not StrEmpty(section) and section ~= "\\" then
                table.insert(files, section)
            end

            j = j + 1
            section = sections[j]
        end

        i = i + 1
        line = lines[i]
    end

    return files
end

function _GCC_Compile(exe, source, type, options)
    if type == "exe" then
        local cmd = exe .. " -c -Wall " .. StrJoin(GetCFlags(), " \"", "\"") .. " " .. SrcDir() .. source

        if TableHas(options, "C++0x") then
            cmd = cmd .. " -std=c++0x"
        end

        cmd = cmd .. StrJoin(GetIncludeDirs(), " \"-I", "\"")
        cmd = cmd .. StrJoin(GetDefinitions(), " \"-D", "\"")

        cmd = cmd .. " -o " .. BuildDir .. source .. ".o"

        if Execute(cmd, false) == 0 then
            return true
        end
    elseif type == "shared" then
        local cmd = exe .. " -c -Wall -fPIC " .. StrJoin(GetCFlags(), " \"", "\"") .. " " .. SrcDir() .. source

        if TableHas(options, "C++0x") then
            cmd = cmd .. " -std=c++0x"
        end

        cmd = cmd .. StrJoin(GetIncludeDirs(), " \"-I", "\"")
        cmd = cmd .. StrJoin(GetDefinitions(), " \"-D", "\"")

        cmd = cmd .. " -o " .. BuildDir .. source .. ".o"

        if Execute(cmd, false) == 0 then
            return true
        end
    elseif type == "static" then
        local cmd = exe .. " -c -Wall -fPIC " .. StrJoin(GetCFlags(), " \"", "\"") .. " " .. SrcDir() .. source

        if TableHas(options, "C++0x") then
            cmd = cmd .. " -std=c++0x"
        end

        cmd = cmd .. StrJoin(GetLibraryDirs(), " \"-L", "\"")
        cmd = cmd .. StrJoin(GetLibraries(), " \"-l", "\"")
        cmd = cmd .. StrJoin(GetIncludeDirs(), " \"-I", "\"")
        cmd = cmd .. StrJoin(GetDefinitions(), " \"-D", "\"")

        cmd = cmd .. " -o " .. BuildDir .. source .. ".o"

        if Execute(cmd, false) == 0 then
            return true
        end
    end

    return false
end

function _GCC_Link(exe, name, type, options)
    if type == "exe" then
        local cmd = exe .. " -Wall -o \"" .. BuildDir .. name .. "\""

        if TableHas(options, "C++0x") then
            cmd = cmd .. " -std=c++0x"
        end

        cmd = cmd .. StrJoin(GetSources(), " \"" .. BuildDir, ".o\"")
        cmd = cmd .. StrJoin(GetLibraries(), " \"-l", "\"")
        cmd = cmd .. StrJoin(GetIncludeDirs(), " \"-I", "\"")
        cmd = cmd .. StrJoin(GetLibraryDirs(), " \"-L", "\"")
        cmd = cmd .. StrJoin(GetDefinitions(), " \"-D", "\"")
        cmd = cmd .. StrJoin(GetLFlags(), " \"", "\"")

        if Execute(cmd, false) == 0 then
            return true
        end
    elseif type == "shared" then
        local cmd = exe .. " -Wall -shared -o \"" .. BuildDir .. name .. "\""

        if TableHas(options, "C++0x") then
            cmd = cmd .. " -std=c++0x"
        end

        cmd = cmd .. StrJoin(GetSources(), " \"" .. BuildDir, ".o\"")
        cmd = cmd .. StrJoin(GetLibraries(), " \"-l", "\"")
        cmd = cmd .. StrJoin(GetIncludeDirs(), " \"-I", "\"")
        cmd = cmd .. StrJoin(GetLibraryDirs(), " \"-L", "\"")
        cmd = cmd .. StrJoin(GetDefinitions(), " \"-D", "\"")
        cmd = cmd .. StrJoin(GetLFlags(), " \"", "\"")

        if Execute(cmd, false) == 0 then
            return true
        end
    elseif type == "static" then
        local cmd = "ar rcs " .. BuildDir .. name

        cmd = cmd .. StrJoin(GetSources(), " \"" .. BuildDir, ".o\"")

        if Execute(cmd, false) == 0 then
            return true
        end
    end

    return false
end

local gccExe = PickExecutable("gcc", "gcc.exe")

if gccExe ~= "" and FindExecutable(gccExe) then
    function gcc_LibraryExists(lib)
        return _GCC_LibraryExists(gccExe, lib)
    end

    function gcc_Deps(file)
        return _GCC_Deps(gccExe, file)
    end

    function gcc_Compile(src, type, options)
        return _GCC_Compile(gccExe, src, type, options)
    end

    function gcc_Link(executable, type, options)
        return _GCC_Link(gccExe, executable, type, options)
    end

    RegisterCompiler("gcc", "C")
end

local gppExe = PickExecutable("g++", "g++.exe")

if gppExe ~= "" and FindExecutable(gppExe) then
    function gpp_LibraryExists(lib)
        return _GCC_LibraryExists(gppExe, lib)
    end

    function gpp_Deps(file)
        return _GCC_Deps(gppExe, file)
    end

    function gpp_Compile(src, type, options)
        return _GCC_Compile(gppExe, src, type, options)
    end

    function gpp_Link(executable, type, options)
        return _GCC_Link(gppExe, executable, type, options)
    end

    RegisterCompiler("gpp", "C++")
end
