#ifndef FUNCTIONS_HPP
#define FUNCTIONS_HPP

namespace Output
{
    void error(string str);
    void status(string str);
    void debug(string str);
    void print(string str, bool newLine = true);
    void warning(string str);
}

namespace Input
{
    string question(string str);
}

namespace Lua
{
    string getString(lua_State *L, int index);
    void pushString(lua_State *L, string str);

    char getChar(lua_State *L, int index);
    
    float getFloat(lua_State *L, int index);

    vector<string> getStringList(lua_State *L, int index);
    void pushStringList(lua_State *L, vector<string> value);

    bool getBool(lua_State *L, int index);
    void pushBool(lua_State *L, bool b);

    void pushInteger(lua_State *L, int i);
    
    void setStrVar(lua_State *L, string name, string value);
    
    void error(lua_State *L, string str =  "Incorrect number of arguments");
}

namespace FS
{
    vector<string> listDir(string directory);

    bool fileExists(string filename);
    bool dirExists(string filename);

    bool mkdir(string dir, bool print = true);
    bool rm(string filename, bool print = true);
    bool mv(string source, string location, bool print = true);
    bool cp(string source, string location, bool print = true);

    bool writeFile(string filename, string data);

    bool requiresCompiling(string source, string object);
}

namespace Str
{
    vector<string> split(string str, char delim);
    string join(vector<string> list, string startDelim, string endDelim);

    string lower(string str);
    string upper(string str);

    bool startsWith(string str, string begin);
}

tuple<int, string> execute(string cmd, bool print = true);

#endif // FUNCTIONS_HPP
