#ifndef PROJECT_HPP
#define PROJECT_HPP

class Project
{
    public:
        Project(lua_State *L);
        ~Project();

        void reset();

        string getCompiledName(string type, string executable);

        void setName(string n);
        string getName();

        void setLanguage(string l);
        string getLanguage();

        void setCompiler(string n);

        void addSource(string filename, bool generated);
        void addHeader(string filename, bool generated);
        void addResource(string filename, bool generated);
        void addPackage(string name, float minVersion, float maxVersion);
        void addLibrary(string name);
        void addIncludeDir(string name);
        void addLibraryDir(string name);
        void addDefinition(string name);
        void addCFlag(string name);
        void addLFlag(string name);
        void addOption(string name, string description, bool defaultValue = true);

        Compiler *getCompiler();
        bool getOption(string name);

        bool executableExists(string name, bool print = true);
        bool packageExists(string name, bool print = true);
        bool libraryExists(string name, bool print = true);
        bool includeDirExists(string name, bool print = true);
        bool libraryDirExists(string name, bool print = true);

        string binDir(string name);
        string libDir(string name);
        string incDir(string name);
        string dataDir(string name);
        string confDir(string name);

        vector<string> sources;
        vector<string> headers;
        vector<string> resources;
        vector<string> libraries;
        vector<string> includeDirs;
        vector<string> libraryDirs;
        vector<string> definitions;
        vector<string> cflags;
        vector<string> lflags;
        map<string, bool> options;

        vector<string> generatedFiles;
        map<string, string> installFiles;

    private:
        string name;
        string language;

        lua_State *luaState;
        Compiler *compiler;
};

#endif // PROJECT_HPP
