#include <fstream>

#include "headers.hpp"

Cache::Cache()
{
    FS::mkdir(".tmake.cache", false);

    read();
}

Cache::~Cache()
{
    write();
}

void Cache::sync()
{
    write();
    read();
}

void Cache::read()
{
    Output::debug("Loading cache...");

    libs.clear();
    ifstream fp1(OS::convertPath(".tmake.cache/libs.ini").c_str(), ifstream::in);
    if (fp1.is_open())
    {
        string line;

        while (!fp1.eof())
        {
            getline(fp1, line);

            string key = line.substr(0, line.find("="));
            string value = line.substr(line.find("=") + 1, line.size());

            libs[key] = value;
        }

        fp1.close();
    }

    incDirs.clear();
    ifstream fp2(OS::convertPath(".tmake.cache/incDirs.ini").c_str(), ifstream::in);
    if (fp2.is_open())
    {
        string line;

        while (!fp2.eof())
        {
            getline(fp2, line);

            string key = line.substr(0, line.find("="));
            string value = line.substr(line.find("=") + 1, line.size());

            incDirs[key] = value;
        }

        fp2.close();
    }

    libDirs.clear();
    ifstream fp3(OS::convertPath(".tmake.cache/libDirs.ini").c_str(), ifstream::in);
    if (fp3.is_open())
    {
        string line;

        while (!fp3.eof())
        {
            getline(fp3, line);

            string key = line.substr(0, line.find("="));
            string value = line.substr(line.find("=") + 1, line.size());

            libDirs[key] = value;
        }

        fp3.close();
    }

    genFiles.clear();
    ifstream fp4(OS::convertPath(".tmake.cache/genFiles.ini").c_str(), ifstream::in);
    if (fp4.is_open())
    {
        string line;

        while (!fp4.eof())
        {
            getline(fp4, line);

            if (!line.empty())
            {
                genFiles.push_back(line);
            }
        }

        fp4.close();
    }

    instFiles.clear();
    ifstream fp5(OS::convertPath(".tmake.cache/instFiles.ini").c_str(), ifstream::in);
    if (fp5.is_open())
    {
        string line;

        while (!fp5.eof())
        {
            getline(fp5, line);

            string key = line.substr(0, line.find("="));
            string value = line.substr(line.find("=") + 1, line.size());

            if (!key.empty() && !value.empty())
            {
                instFiles[key] = value;
            }
        }

        fp5.close();
    }

    options.clear();
    ifstream fp6(OS::convertPath(".tmake.cache/options.ini").c_str(), ifstream::in);
    if (fp6.is_open())
    {
        string line;

        while (!fp6.eof())
        {
            getline(fp6, line);

            string key = line.substr(0, line.find("="));
            string value = line.substr(line.find("=") + 1, line.size());

            if (!key.empty() && !value.empty())
            {
                options[key] = value;
            }
        }

        fp6.close();
    }
}

void Cache::write()
{
    Output::debug("Saving cache...");

    ofstream fp1(OS::convertPath(".tmake.cache/libs.ini").c_str(), ofstream::out);
    map<string, string>::iterator iter1;
    for (iter1 = libs.begin(); iter1 != libs.end(); iter1++)
    {
        fp1 << iter1->first << "=" << iter1->second << "\n";
    }
    fp1.close();

    ofstream fp2(OS::convertPath(".tmake.cache/incDirs.ini").c_str(), ofstream::out);
    map<string, string>::iterator iter2;
    for (iter2 = incDirs.begin(); iter2 != incDirs.end(); iter2++)
    {
        fp2 << iter2->first << "=" << iter2->second << "\n";
    }
    fp2.close();

    ofstream fp3(OS::convertPath(".tmake.cache/libDirs.ini").c_str(), ofstream::out);
    map<string, string>::iterator iter3;
    for (iter3 = libDirs.begin(); iter3 != libDirs.end(); iter3++)
    {
        fp3 << iter3->first << "=" << iter3->second << "\n";
    }
    fp3.close();

    ofstream fp4(OS::convertPath(".tmake.cache/genFiles.ini").c_str(), ofstream::out);
    for (unsigned int i = 0; i < genFiles.size(); i++)
    {
        fp4 << genFiles[i] << "\n";
    }
    fp4.close();

    ofstream fp5(OS::convertPath(".tmake.cache/instFiles.ini").c_str(), ofstream::out);
    map<string, string>::iterator iter5;
    for (iter5 = instFiles.begin(); iter5 != instFiles.end(); iter5++)
    {
        fp5 << iter5->first << "=" << iter5->second << "\n";
    }
    fp5.close();

    ofstream fp6(OS::convertPath(".tmake.cache/options.ini").c_str(), ofstream::out);
    map<string, string>::iterator iter6;
    for (iter6 = options.begin(); iter6 != options.end(); iter6++)
    {
        fp6 << iter6->first << "=" << iter6->second << "\n";
    }
    fp6.close();
}

void Cache::addLib(string name)
{
    libs[name] = "true";
    sync();
}

void Cache::addIncDir(string name)
{
    incDirs[name] = "true";
    sync();
}

void Cache::addLibDir(string name)
{
    libDirs[name] = "true";
    sync();
}

void Cache::addOption(string name, bool value)
{
    if (value)
    {
        options[name] = "true";
    }
    else
    {
        options[name] = "false";
    }

    sync();
}

bool Cache::optionExists(string name)
{
    return !options[name].empty();
}

bool Cache::checkLib(string name)
{
    if (libs[name] == "true")
    {
        return true;
    }

    return false;
}

bool Cache::checkIncDir(string name)
{
    if (incDirs[name] == "true")
    {
        return true;
    }

    return false;
}

bool Cache::checkLibDir(string name)
{
    if (libDirs[name] == "true")
    {
        return true;
    }

    return false;
}

bool Cache::checkOption(string name)
{
    return options[name] == "true";
}

void Cache::setPickString(const vector<string> &choices, string choice)
{
    string choicesStr = Str::join(choices, "|", "") + "|";
    Output::debug(choicesStr + " -> " + choice);

    //picks[choicesStr] = choice;
}

string Cache::pickString(const vector<string> &choices)
{
    string choicesStr = Str::join(choices, "|", "") + "|";
    //return picks[choicesStr];
    return "";
}

void Cache::setGeneratedFiles(vector<string> files)
{
    genFiles = files;
    sync();
}

vector<string> Cache::getGeneratedFiles()
{
    return genFiles;
}

void Cache::setInstallFiles(map<string, string> files)
{
    instFiles = files;
    sync();
}

map<string, string> Cache::getInstallFiles()
{
    return instFiles;
}
