#include "headers.hpp"

Compiler::Compiler(lua_State *L, string n, string l)
{
    luaState = L;
    name = n;
    language = l;

    reset();
}

Compiler::~Compiler()
{
}

void Compiler::reset()
{
    libExistsName = name + "_LibraryExists";
    depsName = name + "_Deps";
    compileName = name + "_Compile";
    linkName = name + "_Link";
}

string Compiler::getName()
{
    return name;
}

string Compiler::getLanguage()
{
    return language;
}

void Compiler::alter(string type, string functionName)
{
    if (type == "LibraryExists")
    {
        libExistsName = functionName;
    }
    else if (type == "Deps")
    {
        depsName = functionName;
    }
    else if (type == "Compile")
    {
        compileName = functionName;
    }
    else if (type == "Link")
    {
        linkName = functionName;
    }
    else
    {
        Lua::error(luaState, "Unknown AlterCompiler type, " + type);
    }
}

bool Compiler::libraryExists(string lib)
{
    lua_getglobal(luaState, libExistsName.c_str());
    Lua::pushString(luaState, lib);
    if (lua_pcall(luaState, 1, 1, 0) == 0)
    {
        return Lua::getBool(luaState, -1);
    }
    else
    {
        string msg = Lua::getString(luaState, lua_gettop(luaState));
        Output::warning("Compiler error, " + msg);
    }

    return false;
}

vector<string> Compiler::dependencies(string filename)
{
    lua_getglobal(luaState, depsName.c_str());
    Lua::pushString(luaState, filename);
    if (lua_pcall(luaState, 1, 1, 0) == 0)
    {
        return Lua::getStringList(luaState, -1);
    }
    else
    {
        string msg = Lua::getString(luaState, lua_gettop(luaState));
        Output::warning("Compiler error, " + msg);
    }

    return vector<string>();
}

bool Compiler::compile(string source, string type, vector<string> options)
{
    lua_getglobal(luaState, compileName.c_str());
    Lua::pushString(luaState, source);
    Lua::pushString(luaState, type);
    Lua::pushStringList(luaState, options);
    if (lua_pcall(luaState, 3, 1, 0) == 0)
    {
        return Lua::getBool(luaState, -1);
    }
    else
    {
        string msg = Lua::getString(luaState, lua_gettop(luaState));
        Output::warning("Compiler error, " + msg);
    }

    return false;
}

bool Compiler::link(string executable, string type, vector<string> options)
{
    lua_getglobal(luaState, linkName.c_str());
    Lua::pushString(luaState, executable);
    Lua::pushString(luaState, type);
    Lua::pushStringList(luaState, options);
    if (lua_pcall(luaState, 3, 1, 0) == 0)
    {
        return Lua::getBool(luaState, -1);
    }
    else
    {
        string msg = Lua::getString(luaState, lua_gettop(luaState));
        Output::warning("Compiler error, " + msg);
    }

    return false;
}

string Compiler::getComputerName(string file, string type)
{
    lua_getglobal(luaState, (name + "_FileName").c_str());
    Lua::pushString(luaState, file);
    Lua::pushString(luaState, type);
    if (lua_pcall(luaState, 2, 1, 0) == 0)
    {
        return Lua::getString(luaState, -1);
    }
    else
    {
        string msg = Lua::getString(luaState, lua_gettop(luaState));
        if (msg != "attempt to call a nil value")
        {
            Output::warning("Compiler error, " + msg);
        }
    }

    return "";
}
