#include <sys/stat.h>
#include <dirent.h>
#include <fstream>

#include "headers.hpp"

bool FS::fileExists(string filename)
{
    struct stat info;
    if (stat(OS::convertPath(filename).c_str(), &info) == 0)
    {
        if (S_ISREG(info.st_mode))
        {
            return true;
        }
    }

    return false;
}

bool FS::dirExists(string filename)
{
    struct stat info;
    if (stat(OS::convertPath(filename).c_str(), &info) == 0)
    {
        if (S_ISDIR(info.st_mode))
        {
            return true;
        }
    }

    return false;
}

vector<string> FS::listDir(string directory)
{
    vector<string> list;

    DIR *dp = opendir(directory.c_str());
    if (dp)
    {
        struct dirent *ep;
        while ((ep = readdir(dp))) // Two brackets to remove warnings
        {
            string name = ep->d_name;
            if (name != "." && name != "..")
            {
                list.push_back(OS::convertPath(directory + "/" + ep->d_name));
            }
        }

        closedir(dp);
    }

    return list;
}

bool FS::mkdir(string dir, bool print)
{
    if (print || tmake->debugMode())
    {
        Output::print("Creating directory: " + dir);
    }

    return OS::mkdir(dir);
}

bool FS::rm(string filename, bool print)
{
    if (print || tmake->debugMode())
    {
        Output::print("Removing: " + filename);
    }

    return OS::rm(filename);
}

bool FS::mv(string source, string location, bool print)
{
    if (print || tmake->debugMode())
    {
        Output::print("Moving: " + source + " -> " + location);
    }

    return OS::mv(source, location);
}

bool FS::cp(string source, string location, bool print)
{
    if (print || tmake->debugMode())
    {
        Output::print("Copying: " + source + " -> " + location);
    }

    return OS::cp(source, location);
}

bool FS::writeFile(string filename, string data)
{
    filename = OS::convertPath(filename);

    Output::print("Writing file: " + filename);

    ofstream fp(filename.c_str(), ofstream::out);
    fp << data;
    fp.close();

    return true;
}

bool FS::requiresCompiling(string source, string object)
{
    if (!fileExists(source) || !fileExists(object))
    {
        return true;
    }

    struct stat sourceStat;
    stat(source.c_str(), &sourceStat);

    struct stat objectStat;
    stat(object.c_str(), &objectStat);

    if (sourceStat.st_mtime > objectStat.st_mtime)
    {
        return true;
    }

    vector<string> deps;

    if (tmake->getProject()->executableExists("tdeps", false))
    {
        tuple<int, string> ret = execute("tdeps " + source + " -I" + tmake->headerDir, false);
        if (get<0>(ret) == 0)
        {
            deps = Str::split(get<1>(ret), '\n');
        }
    }
    else
    {
        deps = tmake->getProject()->getCompiler()->dependencies(source);
    }

    for (unsigned int i = 0; i < deps.size(); i++)
    {
        string dep = deps[i];

        if (fileExists(dep))
        {
            struct stat depStat;
            stat(deps[i].c_str(), &depStat);

            if (depStat.st_mtime > objectStat.st_mtime)
            {
                return true;
            }
        }
    }

    return false;
}
