#include "headers.hpp"

string Lua::getString(lua_State *L, int index)
{
    string result;
    
    if (!lua_isstring(L, index))
    {
        luaL_typerror(L, index, "string");
    }
    
    result = lua_tostring(L, index);
    return result;
}

void Lua::pushString(lua_State *L, string str)
{
    lua_pushstring(L, str.c_str());
}

char Lua::getChar(lua_State *L, int index)
{
    if (!lua_isstring(L, index))
    {
        luaL_typerror(L, index, "char");
    }

    string str = lua_tostring(L, index);
    if (str.size() > 1)
    {
        luaL_typerror(L, index, "char");
    }

    return str[0];
}

float Lua::getFloat(lua_State *L, int index)
{
    float result;
    
    if (!lua_isnumber(L, index))
    {
        luaL_typerror(L, index, "float");
    }
    
    result = lua_tonumber(L, index);
    return result;
}

vector<string> Lua::getStringList(lua_State *L, int index)
{
    vector<string> list;

    if (lua_istable(L, index))
    {
        lua_pushnil(L);
        if (index < 0)
        {
            index -= 1;
        }

        while (lua_next(L, index) != 0)
        {
            list.push_back(Lua::getString(L, -1));
            lua_pop(L, 1);
        }
    }
    else
    {
        luaL_typerror(L, index, "string table");
    }

    return list;
}

void Lua::pushStringList(lua_State *L, vector<string> value)
{
    lua_newtable(L);

    for (unsigned int i = 0; i < value.size(); i++)
    {
        int index = i + 1;
        Lua::pushInteger(L, index);
        Lua::pushString(L, value[i]);
        lua_settable(L, -3);
    }
}

bool Lua::getBool(lua_State *L, int index)
{
    bool result;

    if (!lua_isboolean(L, index))
    {
        luaL_typerror(L, index, "boolean");
    }

    result = lua_toboolean(L, index);
    return result;
}

void Lua::pushBool(lua_State *L, bool b)
{
    lua_pushboolean(L, b);
}

void Lua::pushInteger(lua_State *L, int i)
{
    lua_pushinteger(L, i);
}

void Lua::setStrVar(lua_State *L, string name, string value)
{
    Lua::pushString(L, value);
    lua_setglobal(L, name.c_str());
}

void Lua::error(lua_State *L, string str)
{
    luaL_error(L, str.c_str());
}
