#include "headers.hpp"

Package::Package(string n, lua_State *L)
{
    name = n;
    luaState = L;

    version = -1;
    errorMsg = "";

    Output::debug("Loading package: " + name);
    if (luaL_dofile(luaState, OS::convertPath(OS::packagesDir() + "/" + name + ".lua").c_str()) == 0)
    {
        loadTMakePackage();
    }
    else
    {
        errorMsg = Lua::getString(luaState, lua_gettop(luaState));
    }
}

Package::~Package()
{
}

void Package::loadTMakePackage()
{
    lua_getglobal(luaState, (name + "_Package").c_str());
    if (lua_pcall(luaState, 0, 7, 0) == 0)
    {
        version = Lua::getFloat(luaState, -7);
        libraries = Lua::getStringList(luaState, -6);
        includeDirs = Lua::getStringList(luaState, -5);
        libraryDirs = Lua::getStringList(luaState, -4);
        definitions = Lua::getStringList(luaState, -3);
        cflags = Lua::getStringList(luaState, -2);
        lflags = Lua::getStringList(luaState, -1);
    }
    else
    {
        errorMsg = Lua::getString(luaState, lua_gettop(luaState));
    }
}

bool Package::exists()
{
    return errorMsg.empty();
}

string Package::getErrorMsg()
{
    return errorMsg;
}

bool Package::correctVersion(float min, float max)
{
    if (version == -1)
    {
        return true;
    }

    if (min == -1 && max == -1)
    {
        return true;
    }
    else if (min == -1 && version <= max)
    {
        return true;
    }
    else if (max == -1 && version >= min)
    {
        return true;
    }
    else if (version >= min && version <= max)
    {
        return true;
    }

    return false;
}
