#include <iostream>

#include <QtCore/QFile>
#include <QtCore/QLinkedList>
#include <QtCore/QString>

QLinkedList<QString> includeDirs;

QLinkedList<QString> appendToList(QLinkedList<QString> list1,
                                  QLinkedList<QString> list2)
{
    QLinkedList<QString> newList = list1;

    while (!list2.empty())
    {
        newList.append(list2.takeFirst());
    }

    return newList;
}

QLinkedList<QString> parseDeps(QString filename)
{
    QLinkedList<QString> list;

    QFile fp(filename);
    if (fp.open(QIODevice::Text | QIODevice::ReadOnly))
    {
        while (!fp.atEnd())
        {
            QString line = fp.readLine().trimmed();
            
            if (line.startsWith("#include"))
            {
                QString headerLine = line.remove(0, 8).trimmed();
                headerLine = headerLine.remove(0, 1);
                headerLine = headerLine.remove(headerLine.size() - 1, 1);

                bool found = false;

                QLinkedList<QString>::iterator i;
                for (i = includeDirs.begin(); i != includeDirs.end(); ++i)
                {
                    QString include = *i;
                    if (QFile::exists(include + "/" + headerLine) && !found)
                    {
                        list = appendToList(list, parseDeps(include + "/" + headerLine));
                        list.append(include + "/" + headerLine);
                        found = true;
                    }
                }

                if (!found && QFile::exists(headerLine))
                {
                   list.append(headerLine);
                }
            }
        }
    }

    return list;
}

int main(int argc, char *argv[])
{
    QLinkedList<QString> files;

    for (int i = 1; i < argc; i++)
    {
        QString arg = argv[i];
        if (arg.startsWith("-I"))
        {
            includeDirs.append(arg.remove(0, 2));
        }
        else
        {
            files.append(arg);
        }
    }

    QLinkedList<QString> outputFiles;

    while (!files.empty())
    {
        QString file = files.takeFirst();
        outputFiles = appendToList(outputFiles, parseDeps(file));
    }

    while (!outputFiles.empty())
    {
        QString header = outputFiles.takeFirst();

        std::cout << qPrintable(header) << "\n";

        // Use '\n' because that is all we want to split up the lines
    }
}
